@@ -1,13 +1,9 @@
-from abc import ABCMeta, abstractmethod
-from collections import defaultdict, namedtuple
-
-
 import copy
-
+import os
 import re
 import sys
-import os
-
+from abc import ABC, abstractmethod
+from collections import defaultdict, namedtuple
 
 from diff_cover.command_runner import execute, run_command_for_code
 
@@ -19,16 +15,12 @@ class QualityReporterError(Exception):
     A quality reporter command produced an error.
     """
 
-    pass
 
-
-class BaseViolationReporter:
+class BaseViolationReporter(ABC):
     """
     Query information from a coverage report.
     """
 
-    __metaclass__ = ABCMeta
-
     def __init__(self, name):
         """
         Provide a name for the coverage report, which will be included
@@ -41,7 +33,6 @@ class BaseViolationReporter:
         """
         Return a list of Violations recorded in `src_path`.
         """
-        pass
 
     def measured_lines(self, src_path):
         """
@@ -54,6 +45,8 @@ class BaseViolationReporter:
         The diff reporter generator will then use all changed lines
         provided by the diff.
         """
+        # An existing quality plugin "sqlfluff" depends on this
+        # being not abstract and returning None
         return None
 
     def name(self):
@@ -67,11 +60,9 @@ class BaseViolationReporter:
         return self._name
 
 
-class QualityDriver:
-    __metaclass__ = ABCMeta
-
+class QualityDriver(ABC):
     def __init__(
-        self, name, supported_extensions, command, exit_codes=[0], output_stderr=False
+        self, name, supported_extensions, command, exit_codes=None, output_stderr=False
     ):
         """
         Args:
@@ -98,7 +89,6 @@ class QualityDriver:
             A dict[Str:Violation]
             Violation is a simple named tuple Defined above
         """
-        pass
 
     @abstractmethod
     def installed(self):
@@ -106,7 +96,13 @@ class QualityDriver:
         Method checks if the provided tool is installed.
         Returns: boolean True if installed
         """
-        pass
+
+    def add_driver_args(self, **kwargs):
+        """Inject additional driver related arguments.
+
+        A driver can override the method. By default an exception is raised.
+        """
+        raise ValueError(f"Unsupported argument(s) {kwargs.keys()}")
 
 
 class QualityReporter(BaseViolationReporter):
@@ -114,7 +110,7 @@ class QualityReporter(BaseViolationReporter):
         """
         Args:
             driver (QualityDriver) object that works with the underlying quality tool
-            reports (list[file]) pre-generated reports. If not provided the tool will be run instead.
+            reports (list[file]) pre-generated reports. If not provided the tool will be run instead
             options (str) options to be passed into the command
         """
         super().__init__(driver.name)
@@ -148,7 +144,7 @@ class QualityReporter(BaseViolationReporter):
                 if self.driver_tool_installed is None:
                     self.driver_tool_installed = self.driver.installed()
                 if not self.driver_tool_installed:
-                    raise OSError("{} is not installed".format(self.driver.name))
+                    raise OSError(f"{self.driver.name} is not installed")
                 command = copy.deepcopy(self.driver.command)
                 if self.options:
                     command.append(self.options)
@@ -190,7 +186,7 @@ class RegexBasedDriver(QualityDriver):
         expression,
         command_to_check_install,
         flags=0,
-        exit_codes=[0],
+        exit_codes=None,
     ):
         """
         args:
