@@ -2,19 +2,20 @@
 Classes for querying the information in a test coverage report.
 """
 
+import itertools
+import os
+import os.path
 import re
 from collections import defaultdict
 
-import os
-import itertools
-import posixpath
+from diff_cover import util
 from diff_cover.command_runner import run_command_for_code
 from diff_cover.git_path import GitPathTool
 from diff_cover.violationsreporters.base import (
     BaseViolationReporter,
-    Violation,
-    RegexBasedDriver,
     QualityDriver,
+    RegexBasedDriver,
+    Violation,
 )
 
 
@@ -23,7 +24,7 @@ class XmlCoverageReporter(BaseViolationReporter):
     Query information from a Cobertura|Clover|JaCoCo XML coverage report.
     """
 
-    def __init__(self, xml_roots, src_roots=[""]):
+    def __init__(self, xml_roots, src_roots=None):
         """
         Load the XML coverage report represented
         by the cElementTree with root element `xml_root`.
@@ -35,22 +36,7 @@ class XmlCoverageReporter(BaseViolationReporter):
         # Keys are source file paths, values are output of `violations()`
         self._info_cache = defaultdict(list)
 
-        self._src_roots = src_roots
-
-    @staticmethod
-    def _to_unix_path(path):
-        """
-        Tries to ensure tha the path is a normalized unix path.
-        This seems to be the solution cobertura used....
-        https://github.com/cobertura/cobertura/blob/642a46eb17e14f51272c6962e64e56e0960918af/cobertura/src/main/java/net/sourceforge/cobertura/instrument/ClassPattern.java#L84
-
-        I know of at least one case where this will fail (\\) is allowed in unix paths.
-        But I am taking the bet that this is not common. We deal with source code.
-
-        :param path: string of the path to convert
-        :return: the unix version of that path
-        """
-        return posixpath.normpath(os.path.normcase(path).replace("\\", "/"))
+        self._src_roots = src_roots or [""]
 
     def _get_classes(self, xml_document, src_path):
         """
@@ -70,26 +56,26 @@ class XmlCoverageReporter(BaseViolationReporter):
         # If cwd is `/home/user/work/diff-cover/diff_cover`
         # and src_path is `diff_cover/violations_reporter.py`
         # search for `violations_reporter.py`
-        src_rel_path = self._to_unix_path(GitPathTool.relative_path(src_path))
+        src_rel_path = util.to_unix_path(GitPathTool.relative_path(src_path))
 
         # If cwd is `/home/user/work/diff-cover/diff_cover`
         # and src_path is `other_package/some_file.py`
         # search for `/home/user/work/diff-cover/other_package/some_file.py`
-        src_abs_path = self._to_unix_path(GitPathTool.absolute_path(src_path))
+        src_abs_path = util.to_unix_path(GitPathTool.absolute_path(src_path))
 
         # cobertura sometimes provides the sources for the measurements
         # within it. If we have that we outta use it
         sources = xml_document.findall("sources/source")
         sources = [source.text for source in sources if source.text]
-        classes = [class_tree for class_tree in xml_document.findall(".//class") or []]
+        classes = xml_document.findall(".//class") or []
 
-        classes = (
+        return (
             [
                 clazz
                 for clazz in classes
                 if src_abs_path
                 in [
-                    self._to_unix_path(
+                    util.to_unix_path(
                         os.path.join(source.strip(), clazz.get("filename"))
                     )
                     for source in sources
@@ -98,26 +84,25 @@ class XmlCoverageReporter(BaseViolationReporter):
             or [
                 clazz
                 for clazz in classes
-                if self._to_unix_path(clazz.get("filename")) == src_abs_path
+                if util.to_unix_path(clazz.get("filename")) == src_abs_path
             ]
             or [
                 clazz
                 for clazz in classes
-                if self._to_unix_path(clazz.get("filename")) == src_rel_path
+                if util.to_unix_path(clazz.get("filename")) == src_rel_path
             ]
         )
-        return classes
 
-    def _get_src_path_line_nodes_cobertura(self, xml_document, src_path):
+    def get_src_path_line_nodes_cobertura(self, xml_document, src_path):
         classes = self._get_classes(xml_document, src_path)
 
         if not classes:
             return None
-        else:
-            lines = [clazz.findall("./lines/line") for clazz in classes]
-            return [elem for elem in itertools.chain(*lines)]
+        lines = [clazz.findall("./lines/line") for clazz in classes]
+        return list(itertools.chain(*lines))
 
-    def _get_src_path_line_nodes_clover(self, xml_document, src_path):
+    @staticmethod
+    def get_src_path_line_nodes_clover(xml_document, src_path):
         """
         Return a list of nodes containing line information for `src_path`
         in `xml_document`.
@@ -128,17 +113,20 @@ class XmlCoverageReporter(BaseViolationReporter):
         files = [
             file_tree
             for file_tree in xml_document.findall(".//file")
-            if GitPathTool.relative_path(file_tree.get("path")) == src_path or []
+            if GitPathTool.relative_path(file_tree.get("path")) == src_path
         ]
         if not files:
             return None
-        lines = [file_tree.findall('./line[@type="stmt"]') for file_tree in files]
-        return [elem for elem in itertools.chain(*lines)]
+        lines = []
+        for file_tree in files:
+            lines.append(file_tree.findall('./line[@type="stmt"]'))
+            lines.append(file_tree.findall('./line[@type="cond"]'))
+        return list(itertools.chain(*lines))
 
     def _measured_source_path_matches(self, package_name, file_name, src_path):
         # find src_path in any of the source roots
         if not src_path.endswith(file_name):
-            return None
+            return False
 
         norm_src_path = os.path.normcase(src_path)
         for root in self._src_roots:
@@ -151,8 +139,9 @@ class XmlCoverageReporter(BaseViolationReporter):
                 == norm_src_path
             ):
                 return True
+        return False
 
-    def _get_src_path_line_nodes_jacoco(self, xml_document, src_path):
+    def get_src_path_line_nodes_jacoco(self, xml_document, src_path):
         """
         Return a list of nodes containing line information for `src_path`
         in `xml_document`.
@@ -161,7 +150,7 @@ class XmlCoverageReporter(BaseViolationReporter):
         """
 
         files = []
-        packages = [pkg for pkg in xml_document.findall(".//package")]
+        packages = list(xml_document.findall(".//package"))
         for pkg in packages:
             _files = [
                 _file
@@ -169,14 +158,13 @@ class XmlCoverageReporter(BaseViolationReporter):
                 if self._measured_source_path_matches(
                     pkg.get("name"), _file.get("name"), src_path
                 )
-                or []
             ]
             files.extend(_files)
 
         if not files:
             return None
         lines = [file_tree.findall("./line") for file_tree in files]
-        return [elem for elem in itertools.chain(*lines)]
+        return list(itertools.chain(*lines))
 
     def _cache_file(self, src_path):
         """
@@ -199,21 +187,21 @@ class XmlCoverageReporter(BaseViolationReporter):
             for xml_document in self._xml_roots:
                 if xml_document.findall(".[@clover]"):
                     # see etc/schema/clover.xsd at  https://bitbucket.org/atlassian/clover/src
-                    line_nodes = self._get_src_path_line_nodes_clover(
+                    line_nodes = self.get_src_path_line_nodes_clover(
                         xml_document, src_path
                     )
                     _number = "num"
                     _hits = "count"
                 elif xml_document.findall(".[@name]"):
                     # https://github.com/jacoco/jacoco/blob/master/org.jacoco.report/src/org/jacoco/report/xml/report.dtd
-                    line_nodes = self._get_src_path_line_nodes_jacoco(
+                    line_nodes = self.get_src_path_line_nodes_jacoco(
                         xml_document, src_path
                     )
                     _number = "nr"
                     _hits = "ci"
                 else:
                     # https://github.com/cobertura/web/blob/master/htdocs/xml/coverage-04.dtd
-                    line_nodes = self._get_src_path_line_nodes_cobertura(
+                    line_nodes = self.get_src_path_line_nodes_cobertura(
                         xml_document, src_path
                     )
                     _number = "number"
@@ -294,24 +282,14 @@ pyflakes_driver = RegexBasedDriver(
 
 """
     Report Flake8 violations.
-
-    Flake8 warning/error codes:
-        E***/W***: pycodestyle errors and warnings
-        F***: pyflakes codes
-        C9**: mccabe complexity plugin
-        N8**: pycodestyle-naming plugin
-        T000: flake8-todo plugin
-
-    http://flake8.readthedocs.org/en/latest/warnings.html
 """
 flake8_driver = RegexBasedDriver(
     name="flake8",
     supported_extensions=["py"],
     command=["flake8"],
     # Match lines of the form:
-    # path/to/file.py:328: undefined name '_thing'
-    # path/to/file.py:418: 'random' imported but unused
-    expression=r"^([^:]+):(\d+).*([EWFCNTIBDSQ]\d{3}.*)$",
+    # new_file.py:1:17: E231 whitespace
+    expression=r"^([^:]+):(\d+):(?:\d+): ([a-zA-Z]+\d+.*)$",
     command_to_check_install=["flake8", "--version"],
     # flake8 exit code is 1 if there are violations
     # http://flake8.pycqa.org/en/latest/user/invocation.html
@@ -326,13 +304,32 @@ jshint_driver = RegexBasedDriver(
     command_to_check_install=["jshint", "-v"],
 )
 
-eslint_driver = RegexBasedDriver(
-    name="eslint",
-    supported_extensions=["js"],
-    command=["eslint", "--format=compact"],
-    expression=r"^([^:]+): line (\d+), col \d+, (.*)$",
-    command_to_check_install=["eslint", "-v"],
-)
+
+class EslintDriver(RegexBasedDriver):
+    def __init__(self):
+        super().__init__(
+            name="eslint",
+            supported_extensions=["js"],
+            command=["eslint", "--format=compact"],
+            expression=r"^([^:]+): line (\d+), col \d+, (.*)$",
+            command_to_check_install=["eslint", "-v"],
+        )
+        self.report_root_path = None
+
+    def add_driver_args(self, **kwargs):
+        self.report_root_path = kwargs.pop("report_root_path", None)
+        if kwargs:
+            super().add_driver_args(**kwargs)
+
+    def parse_reports(self, reports):
+        violations_dict = super().parse_reports(reports)
+        if self.report_root_path:
+            keys = list(violations_dict.keys())
+            for key in keys:
+                new_key = os.path.relpath(key, self.report_root_path)
+                violations_dict[new_key] = violations_dict.pop(key)
+        return violations_dict
+
 
 """
     Report pydocstyle violations.
@@ -374,7 +371,7 @@ class PylintDriver(QualityDriver):
             ],
             # Pylint returns bit-encoded exit codes as documented here:
             # https://pylint.readthedocs.io/en/latest/user_guide/run.html
-            # 1 = fatal error, if an error occurred which prevented pylint from doing further processing
+            # 1 = fatal error, occurs if an error prevents pylint from doing further processing
             # 2,4,8,16 = error/warning/refactor/convention message issued
             # 32 = usage error
             [
@@ -405,7 +402,7 @@ class PylintDriver(QualityDriver):
         # Match lines of the form:
         # path/to/file.py:123: [C0111] Missing docstring
         # path/to/file.py:456: [C0111, Foo.bar] Missing docstring
-        self.multi_line_violation_regex = re.compile(r"==(\w|.+):(.*)")
+        self.multi_line_violation_regex = re.compile(r"==((?:\w|\.)+?):\[?(\d+)")
         self.dupe_code_violation_regex = re.compile(r"Similar lines in (\d+) files")
 
     def _process_dupe_code_violation(self, lines, current_line, message):
@@ -458,6 +455,8 @@ class PylintDriver(QualityDriver):
 
                     for violation in files_involved:
                         pylint_src_path, line_number = violation
+                        # pylint might uses windows paths
+                        pylint_src_path = util.to_unix_path(pylint_src_path)
                         # If we're looking for a particular source file,
                         # ignore any other source files.
                         if function_name:
@@ -465,7 +464,7 @@ class PylintDriver(QualityDriver):
                                 pylint_code, function_name, message
                             )
                         else:
-                            error_str = "{}: {}".format(pylint_code, message)
+                            error_str = f"{pylint_code}: {message}"
 
                         violation = Violation(int(line_number), error_str)
                         violations_dict[pylint_src_path].append(violation)
